unit FmxUtils;

interface

uses SysUtils, Windows, Classes, Consts;

type
  EInvalidDest = class(EStreamError);
  EFCantMove = class(EStreamError);

procedure CopyFile(const FileName, DestName: string);
procedure MoveFile(const FileName, DestName: string);
function GetFileSize(const FileName: string): LongInt;
function FileDateTime(const FileName: string): TDateTime;
function HasAttr(const FileName: string; Attr: Word): Boolean;
function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;

implementation

uses Forms, ShellAPI, RtlConsts;

const
  SInvalidDest = 'Destinatia %s nu exista!';
  SFCantMove = 'Nu pot muta fisierul %s .';

procedure CopyFile(const FileName, DestName: string);
var
  CopyBuffer: Pointer; { buffer pentru copiere }
  BytesCopied: Longint;
  Source, Dest: Integer; { handleri de fisiere }
  Len: Integer;
  Destination: TFileName; { manipulator pentru nume de fisier extins }
const
  ChunkSize: Longint = 8192; { copiaza in bucati de 8K }
begin
  Destination := ExpandFileName(DestName); { expandeaza calea de destinatie }
  if HasAttr(Destination, faDirectory) then { daca destinatia este un director... }
  begin
    Len :=  Length(Destination);
    if Destination[Len] = '\' then
      Destination := Destination + ExtractFileName(FileName) { ...cloneaza numele fisierului }
    else
      Destination := Destination + '\' + ExtractFileName(FileName); { ...cloneaza numele fisierului }
  end;
GetMem(CopyBuffer, ChunkSize); { aloca bufferul }
  try
    Source := FileOpen(FileName, fmShareDenyWrite); { deschide fisierul sursa }
    if Source < 0 then raise EFOpenError.CreateFmt(SFOpenError, [FileName]);
    try
      Dest := FileCreate(Destination); { creeaza fis dest.; il suprascrie pe cel existent }
      if Dest < 0 then raise EFCreateError.CreateFmt(SFCreateError, [Destination]);
      try
        repeat
          BytesCopied := FileRead(Source, CopyBuffer^, ChunkSize); { citeste felia de 8K }
          if BytesCopied > 0 then { daca am citit ceva... }
            FileWrite(Dest, CopyBuffer^, BytesCopied); { ...scriem felia }
        until BytesCopied < ChunkSize; { pana cand terminam feliile }
      finally
        FileClose(Dest); { inchide fis detinatie }
      end;
    finally
      FileClose(Source); { inchide fis sursa }
    end;
  finally
    FreeMem(CopyBuffer, ChunkSize); { goleste bufferul }
  end;
end;


{ Procedura MoveFile }
{
  Muta fis specificat in FileName in dir specificat in DestDir. Incearca sa
  redenumeasca fisierul. Daca nu se poate, incearca sa copieze fisierul si sterge
  originalul.
  Arunca o exceptie daca fis sursa e read-only.
}

procedure MoveFile(const FileName, DestName: string);
var
  Destination: string;
begin
  Destination := ExpandFileName(DestName); {  }
  if not RenameFile(FileName, Destination) then { se incearca redenumirea }
  begin
    if HasAttr(FileName, faReadOnly) then  { daca este read-only... }
      raise EFCantMove.Create(Format(SFCantMove, [FileName])); { nu il putem sterge }
      CopyFile(FileName, Destination); { il copiem la destinatie...}
//      DeleteFile(FileName); { ...si stergem originalul }
  end;
end;

{ Functia GetFileSize }
{
  Intoarce marimea fis curent fara a-l deschide. Daca fis nu exista,
  intoarece -1.
}

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  try
    if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
      Result := SearchRec.Size
    else Result := -1;
  finally
    SysUtils.FindClose(SearchRec);
  end;
end;

function FileDateTime(const FileName: string): System.TDateTime;
begin
  Result := FileDateToDateTime(FileAge(FileName));
end;

function HasAttr(const FileName: string; Attr: Word): Boolean;
var
 FileAttr: Integer;
begin
  FileAttr := FileGetAttr(FileName);
  if FileAttr = -1 then FileAttr := 0;
  Result := (FileAttr and Attr) = Attr;
end;

function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
end;

end.
